/*
 * Adapted version
 * Adapted by Peter Verkaik
 * date june 18, 2007
 *
 * Copyright  1999-2002 Celsius Research Ltd. All rights reserved.
 * Do not modify this file.
 */

package stamp.core;

/**
 * Receive bytes from the debugger serial interface.
 *
 * @author Chris Waters
 * @version 1.0 8/28/00
 */
public class Terminal {

  static char temp;
  static boolean tempAvailable;

  /**
   * Returns the last character sent from the terminal. If there are no bytes
   * waiting then blocks until a byte is available.
   *
   * @return the last received character.
   */
  public static char getChar() {
    while (!tempAvailable) {
      if (!internalGetChar()) CPU.delay(1050);  // Delay for 100 msec.
    }
    tempAvailable = false;
    return temp;
  }

  /**
   * Read a stream of bytes from the terminal up to a specified
   * amount of bytes or until a RETURN character (0x0D).
   *
   * @param str Character array to hold the characters
   * @param max The maximum allowed number of characters to read
   * @return the number of characters actually read
   */
  public static int getString(char[] str, int max) {
    int i=0;
    while (i<max){ //read up to max characters from Terminal
      str[i]=getChar();
      if (str[i]=='\r') str[i]='\n'; //replace RETURN by NEWLINE
      if (str[i++]=='\n') break; //stop reading at NEWLINE
    }
    str[i]=0; //make ASCIIZ string
    return i; //return length
  }

  /**
   * Checks if a byte is available into the terminal buffer. If at least
   * one byte is available <code>true</code> is returned.
   */
  public static boolean byteAvailable() {
    if ( tempAvailable )
      return true;
    else
      return internalGetChar();
  }

//============================================================================
// Methods and fields below this point are private.
//============================================================================

  static char[] dataRequestCMD = new char[]{20};

  static boolean internalGetChar() {

    CPU.message(dataRequestCMD,1); //request data
    int r = getByte();
    if ( (r>>>8) != 0) {   // if no data
      CPU.delay(21);       // ...wait 2 msec
      r = getByte();       // ...and try again
      if ((r>>>8) != 0) {
        return false;      // definitely no data
      }
    }
    temp = (char)(r&0x00ff);
    tempAvailable = true;
    return true;
  }

  static native int getByte();
}